var sheetsInitialized = [false, false, false, false, false];

window.onload = function() {
    var spread = new GC.Spread.Sheets.Workbook(document.getElementById("spreadSheet"), { sheetCount: 3 });

    spread.fromJSON(FinancialTemplate);
    spread.options.allowDynamicArray = true;
    spread.setActiveSheetIndex(0);

    var bankingSheet = spread.getSheet(1);
    var expensesSheet = spread.getSheet(2);
    var salesTransactionsSheet = spread.getSheet(3);

    spread.suspendPaint();
    setBankingSheet(spread);
    setExpensesSheet(expensesSheet);
    setSalesTransactionsSheet(salesTransactionsSheet);
    setDashboardSheet(spread);
    sheetsInitialized[0] = true;
    spread.resumePaint();

    spread.bind(GC.Spread.Sheets.Events.ActiveSheetChanged, function(sender, args) {
        switch (args.newSheet.name()) {
            case "Banking":
                (!sheetsInitialized[1]) && setBankingChartLocations(bankingSheet);
                sheetsInitialized[1] = true;
                break;
            case "Expenses":
                (!sheetsInitialized[2]) && setExpensesChartLocations(expensesSheet);
                sheetsInitialized[2] = true;
                break;
            case "SalesTransactions":
                (!sheetsInitialized[3]) && setSalesChartLocations(salesTransactionsSheet);
                sheetsInitialized[3] = true;
                break;
        }
    });

    // Hide Range Template Sheets
    spread.getSheet(4).visible(false);
    spread.getSheet(5).visible(false);

    spread.options.scrollbarMaxAlign = true;
}

function formatDateString(date) {
    return (date.getMonth() + 1) + "/" + date.getDate() + "/" + date.getFullYear();
}

function setBankingSheet(spread) {
    var sheet = spread.getSheet(1);

    // Freeze the top area of the sheet
    sheet.frozenRowCount(16);

    // Set Table data from checkingRegisterData.js
    var tableColumns = [],
        names = ['Date', 'Payee', 'Memo', 'Payment', 'Deposit', 'Balance', 'Type', 'Account', 'Added in Banking'],
        labels = ['Date', 'Payee', 'Memo', 'Payment', 'Deposit', 'Balance', 'Type', 'Account', ' Added in Banking'];
    var table = sheet.tables.add('checkingRegisterTable', 15, 0, checkingRegisterData.data.length, 8, GC.Spread.Sheets.Tables.TableThemes.medium21);
    table.autoGenerateColumns(false);
    names.forEach(function (name, index) {
        var tableColumn = new GC.Spread.Sheets.Tables.TableColumn();
        tableColumn.name(labels[index]);
        tableColumn.dataField(name);
        tableColumns.push(tableColumn);
    });
    table.bind(tableColumns, 'data', checkingRegisterData);
    
    // Convert strings to numbers
    for (var r = 16; r < 16 + checkingRegisterData.data.length; r++) {
        sheet.setValue(r, 0, new Date(sheet.getValue(r, 0)));
        (sheet.getValue(r, 3) != "") && sheet.setValue(r, 3, parseFloat(sheet.getText(r, 3).replace(/,/g,'')));
        (sheet.getValue(r, 4) != "") && sheet.setValue(r, 4, parseFloat(sheet.getText(r, 4).replace(/,/g,'')));
        (sheet.getValue(r, 5) != "") && sheet.setValue(r, 5, parseFloat(sheet.getText(r, 5).replace(/,/g,'')));
    }
    // Format currency
    sheet.getRange(16, 3, checkingRegisterData.data.length, 3).formatter('_($* #,##0.00_);_($* (#,##0.00);_($* "-"??_);_(@_)');

    // Create sorted combined data for line chart
    sheet.setText(0, 9, "Date");
    sheet.setText(0, 10, "Date String");
    sheet.setText(0, 11, "Total");
    // Sort the dates and get all unique values by millisecond
    sheet.setFormula(1, 9, "SORT(UNIQUE(checkingRegisterTable[[#Data],[Date]]))");
    sheet.setFormula(1, 11, "SUMIF(checkingRegisterTable[[#Data],[Date]],$J2,checkingRegisterTable[[#Data],[Balance]]");
    startRange = new GC.Spread.Sheets.Range(1, 11, 1, 1);
    wholeRange = new GC.Spread.Sheets.Range(1, 11, 25, 1);
    sheet.fillAuto(startRange, wholeRange, {
       fillType: GC.Spread.Sheets.Fill.FillType.auto,
       series: GC.Spread.Sheets.Fill.FillSeries.column,
       fillDirection:GC.Spread.Sheets.Fill.FillDirection.down
    });

    // Create a separate column for formatting those unique sorted millisecond values as strings
    for (var r = 1; r < 26; r++) {
        sheet.setValue(r, 10, formatDateString(sheet.getValue(r, 9)));
    }

    sheet.setText(0, 13, "Type");
    sheet.setText(0, 14, "Payment");
    sheet.setFormula(1, 13, "SORT(UNIQUE(checkingRegisterTable[[#Data],[Type]]))");
    sheet.setFormula(1, 14, "SUMIF(checkingRegisterTable[[#Data],[Type]],$N2,checkingRegisterTable[[#Data],[Payment]])");
    var startRange = new GC.Spread.Sheets.Range(1, 14, 1, 1);
    var wholeRange = new GC.Spread.Sheets.Range(1, 14, 10, 1);
    sheet.fillAuto(startRange, wholeRange, {
        fillType: GC.Spread.Sheets.Fill.FillType.auto,
        series: GC.Spread.Sheets.Fill.FillSeries.column,
        fillDirection:GC.Spread.Sheets.Fill.FillDirection.down
    });

    // Create range template
    var templateSheet = spread.getSheetFromName("CheckingTemplate");
    var cellType = new GC.Spread.Sheets.CellTypes.RangeTemplate(templateSheet);
    sheet.setCellType(0, 0, cellType, GC.Spread.Sheets.SheetArea.viewport);

    // Bind data for the range template
    var balanceInfo = {
        CurrentBalance: sheet.getValue(16, 5)
    };
    // The value for the cell is used as the data source for the range template
    sheet.setValue(0, 0, balanceInfo);

    // Hide the chart data columns
    for (var c = 9; c <= 14; c++) {
        sheet.setColumnVisible(c, false);
    }

    // Create charts
    var lineChart = sheet.charts.add('line', GC.Spread.Sheets.Charts.ChartType.line, 0, 0, 100, 100, "K1:L26");
    var lineTitle = lineChart.title();
    lineTitle.text = "Balance Over Time";
    lineChart.title(lineTitle);
    var lineLegend = lineChart.legend();
    lineLegend.visible = false;
    lineChart.legend(lineLegend);
    lineChart.ignoreHidden(false);
    lineChart.useAnimation(true);

    var pieChart = sheet.charts.add('pie', GC.Spread.Sheets.Charts.ChartType.pie, 0, 100, 100, 100, "N1:O11");
    var pieTitle = pieChart.title();
    pieTitle.text = "Payments by Type";
    pieChart.title(pieTitle);
    var pieLegend = pieChart.legend();
    pieLegend.position = GC.Spread.Sheets.Charts.LegendPosition.right;
    pieChart.legend(pieLegend);
    pieChart.ignoreHidden(false);
    pieChart.useAnimation(true);
}

function setBankingChartLocations(sheet) {
    var lineChart = sheet.charts.all()[0];
    var lineStartCell = sheet.getCellRect(5, 0);
    var lineEndCell = sheet.getCellRect(15, 5);
    lineChart.x(lineStartCell.x);
    lineChart.y(lineStartCell.y);
    lineChart.width(lineEndCell.x - lineStartCell.x);
    lineChart.height(lineEndCell.y - lineStartCell.y);
 
 
    var pieChart = sheet.charts.all()[1];
    var pieStartCell = sheet.getCellRect(0, 5);
    var pieEndCell = sheet.getCellRect(15, 8);
    pieChart.x(pieStartCell.x);
    pieChart.y(pieStartCell.y);
    pieChart.width(pieEndCell.x - pieStartCell.x);
    pieChart.height(pieEndCell.y - pieStartCell.y);   
}

function setExpensesSheet(sheet) {
    // Freeze the top area of the sheet
    sheet.frozenRowCount(16);

    // Set Table data from expenses.js
    var tableColumns = [],
        names = ['Date', 'Type', 'Payee', 'Category', 'SubCategory','Total'],
        labels = ['Date', 'Expense Type', 'Payee', 'Category', 'Sub Category', 'Total'];
    var table = sheet.tables.add('expensesTable', 15, 0, expenses.data.length, 6, GC.Spread.Sheets.Tables.TableThemes.medium21);
    table.autoGenerateColumns(false);
    names.forEach(function (name, index) {
        var tableColumn = new GC.Spread.Sheets.Tables.TableColumn();
        tableColumn.name(labels[index]);
        tableColumn.dataField(name);
        tableColumns.push(tableColumn);
    });
    table.bind(tableColumns, 'data', expenses);
    // Convert Date string to milliseconds for sorting
    for (var r = 16; r < 16 + expenses.data.length; r++) {
        sheet.setValue(r, 0, new Date(sheet.getValue(r, 0)));
        (sheet.getValue(r, 5) != "") && sheet.setValue(r, 5, parseFloat(sheet.getText(r, 5).replace(/,/g,'')));
    }

    // Format currency
    sheet.getRange(16, 5, expenses.data.length, 1).formatter('_($* #,##0.00_);_($* (#,##0.00);_($* "-"??_);_(@_)');

    // Create sorted combined data for doughnut chart
    //applyTableStyleForRange(sheet, 0, 7, 5, 2);
    sheet.setText(0, 7, "Category");
    sheet.setText(0, 8, "Total");
    sheet.setFormula(1, 7, "SORT(UNIQUE(expensesTable[[#Data],[Category]]))");
    sheet.setFormula(1, 8, "SUMIF(expensesTable[[#Data],[Category]],$H2,expensesTable[[#Data],[Total]])");
    var startRange = new GC.Spread.Sheets.Range(1, 8, 1, 1);
    var wholeRange = new GC.Spread.Sheets.Range(1, 8, 4, 1);
    sheet.fillAuto(startRange, wholeRange, {
        fillType: GC.Spread.Sheets.Fill.FillType.auto,
        series: GC.Spread.Sheets.Fill.FillSeries.column,
        fillDirection:GC.Spread.Sheets.Fill.FillDirection.down
    });

    // Create sorted combined data for line chart
    sheet.setText(6, 7, "Date");
    sheet.setText(6, 8, "Date String");
    sheet.setText(6, 9, "Total");
    // Sort the dates and get all unique values by millisecond
    sheet.setFormula(7, 7, "SORT(UNIQUE(expensesTable[[#Data],[Date]]))");
    sheet.setFormula(7, 9, "SUMIF(expensesTable[[#Data],[Date]],$I8,expensesTable[[#Data],[Total]]");
    startRange = new GC.Spread.Sheets.Range(7, 9, 1, 1);
    wholeRange = new GC.Spread.Sheets.Range(7, 9, 33, 1);
    sheet.fillAuto(startRange, wholeRange, {
       fillType: GC.Spread.Sheets.Fill.FillType.auto,
       series: GC.Spread.Sheets.Fill.FillSeries.column,
       fillDirection:GC.Spread.Sheets.Fill.FillDirection.down
    });
    // Create a separate column for formatting those unique sorted millisecond values as strings
    for (var r = 7; r < 40; r++) {
        sheet.setValue(r, 8, formatDateString(sheet.getValue(r, 7)));
    } 

    // Hide the chart data columns
    for (var c = 7; c <= 9; c++) {
        sheet.setColumnVisible(c, false);
    }

    // Create charts
    var doughnutChart = sheet.charts.add('doughnut', GC.Spread.Sheets.Charts.ChartType.doughnut, 0, 0, 100, 100, "H1:I5");
    var doughnutTitle = doughnutChart.title();
    doughnutTitle.text = "Expenses";
    doughnutChart.title(doughnutTitle);
    var doughnutLegend = doughnutChart.legend();
    doughnutLegend.position = GC.Spread.Sheets.Charts.LegendPosition.right;
    doughnutChart.legend(doughnutLegend);
    doughnutChart.ignoreHidden(false);
    doughnutChart.useAnimation(true);

    var lineChart = sheet.charts.add('line', GC.Spread.Sheets.Charts.ChartType.line, 100, 0, 100, 100, "I7:J40");
    var lineTitle = lineChart.title();
    lineTitle.text = "Daily Expenses";
    lineChart.title(lineTitle);
    var lineLegend = lineChart.legend();
    lineLegend.visible = false;
    lineChart.legend(lineLegend);
    lineChart.ignoreHidden(false);
    lineChart.useAnimation(true);
}

function setExpensesChartLocations(sheet) {
    var doughnutChart = sheet.charts.all()[0];
    var doughnutStartCell = sheet.getCellRect(0, 0);
    var doughnutEndCell = sheet.getCellRect(15, 3);
    doughnutChart.x(doughnutStartCell.x);
    doughnutChart.y(doughnutStartCell.y);
    doughnutChart.width(doughnutEndCell.x - doughnutStartCell.x);
    doughnutChart.height(doughnutEndCell.y - doughnutStartCell.y);

    var lineChart = sheet.charts.all()[1];
    var lineStartCell = sheet.getCellRect(0, 3);
    var lineEndCell = sheet.getCellRect(15, 6);
    lineChart.x(lineStartCell.x);
    lineChart.y(lineStartCell.y);
    lineChart.width(lineEndCell.x - lineStartCell.x);
    lineChart.height(lineEndCell.y - lineStartCell.y);    
}

function setSalesTransactionsSheet(sheet) {
    // Freeze the top area of the sheet
    sheet.frozenRowCount(16);

    // Set Table data from expenses.js
    var tableColumns = [],
        names = ['Date', 'Type', 'Customer', 'Balance', 'Total', 'Status'],
        labels = ['Date', 'Type', 'Customer', 'Balance', 'Total', 'Status'];
    var table = sheet.tables.add('salesTransactionsTable', 15, 0, salesTransactions.data.length, names.length, GC.Spread.Sheets.Tables.TableThemes.medium21);
    table.autoGenerateColumns(false);
    names.forEach(function (name, index) {
        var tableColumn = new GC.Spread.Sheets.Tables.TableColumn();
        tableColumn.name(labels[index]);
        tableColumn.dataField(name);
        tableColumns.push(tableColumn);
    });
    table.bind(tableColumns, 'data', salesTransactions);

    // Convert Date string to milliseconds for sorting
    for (var r = 16; r < 16 + salesTransactions.data.length; r++) {
        sheet.setValue(r, 0, new Date(sheet.getValue(r, 0)));
        (sheet.getValue(r, 3) != "") && sheet.setValue(r, 3, parseFloat(sheet.getText(r, 3).replace(/,/g,'')));
        (sheet.getValue(r, 4) != "") && sheet.setValue(r, 4, parseFloat(sheet.getText(r, 4).replace(/,/g,'')));
    }

    // Format currency
    sheet.getRange(16, 3, salesTransactions.data.length, 2).formatter('_($* #,##0.00_);_($* (#,##0.00);_($* "-"??_);_(@_)');

    // Create sorted combined data for line chart
    sheet.setText(0, 7, "Date");
    sheet.setText(0, 8, "Date String");
    sheet.setText(0, 9, "Total");
    // Sort the dates and get all unique values by millisecond
    sheet.setFormula(1, 7, "SORT(UNIQUE(salesTransactionsTable[[#Data],[Date]]))");
    sheet.setFormula(1, 9, "SUMIF(salesTransactionsTable[[#Data],[Date]],$I2,salesTransactionsTable[[#Data],[Total]]");
    startRange = new GC.Spread.Sheets.Range(1, 9, 1, 1);
    wholeRange = new GC.Spread.Sheets.Range(1, 9, 24, 1);
    sheet.fillAuto(startRange, wholeRange, {
       fillType: GC.Spread.Sheets.Fill.FillType.auto,
       series: GC.Spread.Sheets.Fill.FillSeries.column,
       fillDirection:GC.Spread.Sheets.Fill.FillDirection.down
    });

    // Create a separate column for formatting those unique sorted millisecond values as strings
    for (var r = 1; r < 24; r++) {
        sheet.setValue(r, 8, formatDateString(sheet.getValue(r, 7)));
    } 

    // Hide the chart data columns
    for (var c = 7; c <= 9; c++) {
        sheet.setColumnVisible(c, false);
    }

    // Create charts
    var columnChart = sheet.charts.add('column', GC.Spread.Sheets.Charts.ChartType.column, 0, 0, 100, 100, "I2:J24");
    var columnTitle = columnChart.title();
    columnTitle.text = "Daily Sales Transactions";
    columnChart.title(columnTitle);
    var columnLegend = columnChart.legend();
    columnLegend.visible = false;
    columnChart.legend(columnLegend);
    columnChart.ignoreHidden(false);
    columnChart.useAnimation(true);
}

function setSalesChartLocations(sheet) {
    var columnChart = sheet.charts.all()[0];
    var columnStartCell = sheet.getCellRect(0, 0);
    var columnEndCell = sheet.getCellRect(15, 6);
    columnChart.x(columnStartCell.x);
    columnChart.y(columnStartCell.y);
    columnChart.width(columnEndCell.x - columnStartCell.x);
    columnChart.height(columnEndCell.y - columnStartCell.y); 
}

function setDashboardSheet(spread) {
    var sheet = spread.getSheet(0);

    // Create range template
    var templateSheet = spread.getSheetFromName("DashboardBalanceTemplate");
    var cellType = new GC.Spread.Sheets.CellTypes.RangeTemplate(templateSheet);
    sheet.setCellType(1, 8, cellType, GC.Spread.Sheets.SheetArea.viewport);

    // Bind data for the range template
    var balanceInfo = {
        CurrentBalance: spread.getSheet(1).getValue(16, 5)
    };
    // The value for the cell is used as the data source for the range template
    sheet.setValue(1, 8, balanceInfo);

    // Expenses Doughnut Chart
    var doughnutStartCell = sheet.getCellRect(1, 1);
    var doughnutEndCell = sheet.getCellRect(16, 7);
    var doughnutChart = sheet.charts.add('doughnut', GC.Spread.Sheets.Charts.ChartType.doughnut, doughnutStartCell.x, doughnutStartCell.y, doughnutEndCell.x - doughnutStartCell.x, doughnutEndCell.y - doughnutStartCell.y, "Expenses!H1:I5");
    var doughnutTitle = doughnutChart.title();
    doughnutTitle.text = "Expenses";
    doughnutChart.title(doughnutTitle);
    var doughnutLegend = doughnutChart.legend();
    doughnutLegend.position = GC.Spread.Sheets.Charts.LegendPosition.right;
    doughnutChart.legend(doughnutLegend);
    doughnutChart.ignoreHidden(false);
    doughnutChart.useAnimation(true);

    // Daily Expenses Line Chart
    var lineStartCell = sheet.getCellRect(17, 1);
    var lineEndCell = sheet.getCellRect(32, 7);
    var lineChart = sheet.charts.add('line', GC.Spread.Sheets.Charts.ChartType.line, lineStartCell.x, lineStartCell.y, lineEndCell.x - lineStartCell.x, lineEndCell.y - lineStartCell.y, "Expenses!I7:J40");
    var lineTitle = lineChart.title();
    lineTitle.text = "Daily Expenses";
    lineChart.title(lineTitle);
    var lineLegend = lineChart.legend();
    lineLegend.visible = false;
    lineChart.legend(lineLegend);
    lineChart.ignoreHidden(false);
    lineChart.useAnimation(true);

    // Sales Transactions Column Chart
    var columnStartCell = sheet.getCellRect(17, 8);
    var columnEndCell = sheet.getCellRect(32, 14);
    var columnChart = sheet.charts.add('column', GC.Spread.Sheets.Charts.ChartType.column, columnStartCell.x, columnStartCell.y, columnEndCell.x - columnStartCell.x, columnEndCell.y - columnStartCell.y, "SalesTransactions!I2:J24");
    var columnTitle = columnChart.title();
    columnTitle.text = "Daily Sales Transactions";
    columnChart.title(columnTitle);
    var columnLegend = columnChart.legend();
    columnLegend.visible = false;
    columnChart.legend(columnLegend);
    columnChart.ignoreHidden(false);
    columnChart.useAnimation(true);
}